@@ -32,6 +32,11 @@ struct lxc_handler;
 struct lxc_conf;
 struct lxc_list;
 
+typedef enum {
+	CGFS,
+	CGMANAGER,
+} cgroup_driver_t;
+
 struct cgroup_ops {
 	const char *name;
 
@@ -51,6 +56,7 @@ struct cgroup_ops {
 	bool (*mount_cgroup)(void *hdata, const char *root, int type);
 	int (*nrtasks)(void *hdata);
 	void (*disconnect)(void);
+	cgroup_driver_t driver;
 };
 
 extern bool cgroup_attach(const char *name, const char *lxcpath, pid_t pid);
@@ -72,5 +78,6 @@ extern const char *cgroup_get_cgroup(struct lxc_handler *handler, const char *su
 extern const char *cgroup_canonical_path(struct lxc_handler *handler);
 extern bool cgroup_unfreeze(struct lxc_handler *handler);
 extern void cgroup_disconnect(void);
+extern cgroup_driver_t cgroup_driver(void);
 
 #endif
