@@ -72,10 +72,10 @@ static inline void __lxc_list_add(struct lxc_list *new,
 				  struct lxc_list *prev,
 				  struct lxc_list *next)
 {
-	next->prev = new;
-	new->next = next;
-	new->prev = prev;
-	prev->next = new;
+        next->prev = new;
+        new->next = next;
+        new->prev = prev;
+        prev->next = new;
 }
 
 static inline void lxc_list_add(struct lxc_list *head, struct lxc_list *list)
@@ -99,4 +99,15 @@ static inline void lxc_list_del(struct lxc_list *list)
 	prev->next = next;
 }
 
+static inline int lxc_list_len(struct lxc_list *list)
+{
+	 int i = 0;
+	 struct lxc_list *iter;
+	 lxc_list_for_each(iter, list) {
+		i++;
+	 }
+
+	 return i;
+}
+
 #endif
