@@ -36,7 +36,7 @@
 #include "log.h"
 #include "caps.h"
 #include "error.h"
-#include "utils.h"
+#include "initutils.h"
 
 lxc_log_define(lxc_init, lxc);
 
@@ -104,9 +104,6 @@ int main(int argc, char *argv[])
 		}
 	}
 
-	if (lxc_caps_init())
-		exit(EXIT_FAILURE);
-
 	err = lxc_log_init(name, name ? NULL : "none", logpriority,
 			   basename(argv[0]), quiet, lxcpath);
 	if (err < 0)
@@ -168,9 +165,6 @@ int main(int argc, char *argv[])
 
 	lxc_setup_fs();
 
-	if (lxc_caps_reset())
-		exit(EXIT_FAILURE);
-
 	pid = fork();
 
 	if (pid < 0)
@@ -215,6 +209,7 @@ int main(int argc, char *argv[])
 		case 0:
 			break;
 
+		case SIGPWR:
 		case SIGTERM:
 			if (!shutdown) {
 				shutdown = 1;
