@@ -69,6 +69,11 @@
 
 lxc_log_define(lxc_utils, lxc);
 
+/*
+ * if path is btrfs, tries to remove it and any subvolumes beneath it
+ */
+extern bool btrfs_try_remove_subvol(const char *path);
+
 static int _recursive_rmdir(char *dirname, dev_t pdev,
 			    const char *exclude, int level, bool onedev)
 {
@@ -127,11 +132,16 @@ static int _recursive_rmdir(char *dirname, dev_t pdev,
 		ret = lstat(pathname, &mystat);
 		if (ret) {
 			ERROR("%s: failed to stat %s", __func__, pathname);
-			failed=1;
+			failed = 1;
 			continue;
 		}
-		if (onedev && mystat.st_dev != pdev)
+		if (onedev && mystat.st_dev != pdev) {
+			/* TODO should we be checking /proc/self/mountinfo for
+			 * pathname and not doing this if found? */
+			if (btrfs_try_remove_subvol(pathname))
+				INFO("Removed btrfs subvolume at %s\n", pathname);
 			continue;
+		}
 		if (S_ISDIR(mystat.st_mode)) {
 			if (_recursive_rmdir(pathname, pdev, exclude, level+1, onedev) < 0)
 				failed=1;
@@ -143,11 +153,9 @@ static int _recursive_rmdir(char *dirname, dev_t pdev,
 		}
 	}
 
-	if (rmdir(dirname) < 0) {
-		if (!hadexclude) {
-			ERROR("%s: failed to delete %s", __func__, dirname);
-			failed=1;
-		}
+	if (rmdir(dirname) < 0 && !btrfs_try_remove_subvol(dirname) && !hadexclude) {
+		ERROR("%s: failed to delete %s", __func__, dirname);
+		failed=1;
 	}
 
 	ret = closedir(dir);
@@ -196,43 +204,6 @@ extern int lxc_rmdir_onedev(char *path, const char *exclude)
 	return _recursive_rmdir(path, mystat.st_dev, exclude, 0, onedev);
 }
 
-static int mount_fs(const char *source, const char *target, const char *type)
-{
-	/* the umount may fail */
-	if (umount(target))
-		WARN("failed to unmount %s : %s", target, strerror(errno));
-
-	if (mount(source, target, type, 0, NULL)) {
-		ERROR("failed to mount %s : %s", target, strerror(errno));
-		return -1;
-	}
-
-	DEBUG("'%s' mounted on '%s'", source, target);
-
-	return 0;
-}
-
-extern void lxc_setup_fs(void)
-{
-	if (mount_fs("proc", "/proc", "proc"))
-		INFO("failed to remount proc");
-
-	/* if we can't mount /dev/shm, continue anyway */
-	if (mount_fs("shmfs", "/dev/shm", "tmpfs"))
-		INFO("failed to mount /dev/shm");
-
-	/* If we were able to mount /dev/shm, then /dev exists */
-	/* Sure, but it's read-only per config :) */
-	if (access("/dev/mqueue", F_OK) && mkdir("/dev/mqueue", 0666)) {
-		DEBUG("failed to create '/dev/mqueue'");
-		return;
-	}
-
-	/* continue even without posix message queue support */
-	if (mount_fs("mqueue", "/dev/mqueue", "mqueue"))
-		INFO("failed to mount /dev/mqueue");
-}
-
 /* borrowed from iproute2 */
 extern int get_u16(unsigned short *val, const char *arg, int base)
 {
@@ -275,195 +246,6 @@ extern int mkdir_p(const char *dir, mode_t mode)
 	return 0;
 }
 
-extern void remove_trailing_slashes(char *p)
-{
-	int l = strlen(p);
-	while (--l >= 0 && (p[l] == '/' || p[l] == '\n'))
-		p[l] = '\0';
-}
-
-static char *copy_global_config_value(char *p)
-{
-	int len = strlen(p);
-	char *retbuf;
-
-	if (len < 1)
-		return NULL;
-	if (p[len-1] == '\n') {
-		p[len-1] = '\0';
-		len--;
-	}
-	retbuf = malloc(len+1);
-	if (!retbuf)
-		return NULL;
-	strcpy(retbuf, p);
-	return retbuf;
-}
-
-#define DEFAULT_VG "lxc"
-#define DEFAULT_THIN_POOL "lxc"
-#define DEFAULT_ZFSROOT "lxc"
-
-const char *lxc_global_config_value(const char *option_name)
-{
-	static const char * const options[][2] = {
-		{ "lxc.bdev.lvm.vg",        DEFAULT_VG      },
-		{ "lxc.bdev.lvm.thin_pool", DEFAULT_THIN_POOL },
-		{ "lxc.bdev.zfs.root",      DEFAULT_ZFSROOT },
-		{ "lxc.lxcpath",            NULL            },
-		{ "lxc.default_config",     NULL            },
-		{ "lxc.cgroup.pattern",     NULL            },
-		{ "lxc.cgroup.use",         NULL            },
-		{ NULL, NULL },
-	};
-
-	/* placed in the thread local storage pool for non-bionic targets */
-#ifdef HAVE_TLS
-	static __thread const char *values[sizeof(options) / sizeof(options[0])] = { 0 };
-#else
-	static const char *values[sizeof(options) / sizeof(options[0])] = { 0 };
-#endif
-
-	/* user_config_path is freed as soon as it is used */
-	char *user_config_path = NULL;
-
-	/*
-	 * The following variables are freed at bottom unconditionally.
-	 * So NULL the value if it is to be returned to the caller
-	 */
-	char *user_default_config_path = NULL;
-	char *user_lxc_path = NULL;
-	char *user_cgroup_pattern = NULL;
-
-	if (geteuid() > 0) {
-		const char *user_home = getenv("HOME");
-		if (!user_home)
-			user_home = "/";
-
-		user_config_path = malloc(sizeof(char) * (22 + strlen(user_home)));
-		user_default_config_path = malloc(sizeof(char) * (26 + strlen(user_home)));
-		user_lxc_path = malloc(sizeof(char) * (19 + strlen(user_home)));
-
-		sprintf(user_config_path, "%s/.config/lxc/lxc.conf", user_home);
-		sprintf(user_default_config_path, "%s/.config/lxc/default.conf", user_home);
-		sprintf(user_lxc_path, "%s/.local/share/lxc/", user_home);
-		user_cgroup_pattern = strdup("lxc/%n");
-	}
-	else {
-		user_config_path = strdup(LXC_GLOBAL_CONF);
-		user_default_config_path = strdup(LXC_DEFAULT_CONFIG);
-		user_lxc_path = strdup(LXCPATH);
-		user_cgroup_pattern = strdup(DEFAULT_CGROUP_PATTERN);
-	}
-
-	const char * const (*ptr)[2];
-	size_t i;
-	char buf[1024], *p, *p2;
-	FILE *fin = NULL;
-
-	for (i = 0, ptr = options; (*ptr)[0]; ptr++, i++) {
-		if (!strcmp(option_name, (*ptr)[0]))
-			break;
-	}
-	if (!(*ptr)[0]) {
-		free(user_config_path);
-		free(user_default_config_path);
-		free(user_lxc_path);
-		free(user_cgroup_pattern);
-		errno = EINVAL;
-		return NULL;
-	}
-
-	if (values[i]) {
-		free(user_config_path);
-		free(user_default_config_path);
-		free(user_lxc_path);
-		free(user_cgroup_pattern);
-		return values[i];
-	}
-
-	fin = fopen_cloexec(user_config_path, "r");
-	free(user_config_path);
-	if (fin) {
-		while (fgets(buf, 1024, fin)) {
-			if (buf[0] == '#')
-				continue;
-			p = strstr(buf, option_name);
-			if (!p)
-				continue;
-			/* see if there was just white space in front
-			 * of the option name
-			 */
-			for (p2 = buf; p2 < p; p2++) {
-				if (*p2 != ' ' && *p2 != '\t')
-					break;
-			}
-			if (p2 < p)
-				continue;
-			p = strchr(p, '=');
-			if (!p)
-				continue;
-			/* see if there was just white space after
-			 * the option name
-			 */
-			for (p2 += strlen(option_name); p2 < p; p2++) {
-				if (*p2 != ' ' && *p2 != '\t')
-					break;
-			}
-			if (p2 < p)
-				continue;
-			p++;
-			while (*p && (*p == ' ' || *p == '\t')) p++;
-			if (!*p)
-				continue;
-
-			if (strcmp(option_name, "lxc.lxcpath") == 0) {
-				free(user_lxc_path);
-				user_lxc_path = copy_global_config_value(p);
-				remove_trailing_slashes(user_lxc_path);
-				values[i] = user_lxc_path;
-				user_lxc_path = NULL;
-				goto out;
-			}
-
-			values[i] = copy_global_config_value(p);
-			goto out;
-		}
-	}
-	/* could not find value, use default */
-	if (strcmp(option_name, "lxc.lxcpath") == 0) {
-		remove_trailing_slashes(user_lxc_path);
-		values[i] = user_lxc_path;
-		user_lxc_path = NULL;
-	}
-	else if (strcmp(option_name, "lxc.default_config") == 0) {
-		values[i] = user_default_config_path;
-		user_default_config_path = NULL;
-	}
-	else if (strcmp(option_name, "lxc.cgroup.pattern") == 0) {
-		values[i] = user_cgroup_pattern;
-		user_cgroup_pattern = NULL;
-	}
-	else
-		values[i] = (*ptr)[1];
-
-	/* special case: if default value is NULL,
-	 * and there is no config, don't view that
-	 * as an error... */
-	if (!values[i])
-		errno = 0;
-
-out:
-	if (fin)
-		fclose(fin);
-
-	free(user_cgroup_pattern);
-	free(user_default_config_path);
-	free(user_lxc_path);
-
-	return values[i];
-}
-
 char *get_rundir()
 {
 	char *rundir;
@@ -669,50 +451,6 @@ const char** lxc_va_arg_list_to_argv_const(va_list ap, size_t skip)
 	return (const char**)lxc_va_arg_list_to_argv(ap, skip, 0);
 }
 
-FILE *fopen_cloexec(const char *path, const char *mode)
-{
-	int open_mode = 0;
-	int step = 0;
-	int fd;
-	int saved_errno = 0;
-	FILE *ret;
-
-	if (!strncmp(mode, "r+", 2)) {
-		open_mode = O_RDWR;
-		step = 2;
-	} else if (!strncmp(mode, "r", 1)) {
-		open_mode = O_RDONLY;
-		step = 1;
-	} else if (!strncmp(mode, "w+", 2)) {
-		open_mode = O_RDWR | O_TRUNC | O_CREAT;
-		step = 2;
-	} else if (!strncmp(mode, "w", 1)) {
-		open_mode = O_WRONLY | O_TRUNC | O_CREAT;
-		step = 1;
-	} else if (!strncmp(mode, "a+", 2)) {
-		open_mode = O_RDWR | O_CREAT | O_APPEND;
-		step = 2;
-	} else if (!strncmp(mode, "a", 1)) {
-		open_mode = O_WRONLY | O_CREAT | O_APPEND;
-		step = 1;
-	}
-	for (; mode[step]; step++)
-		if (mode[step] == 'x')
-			open_mode |= O_EXCL;
-	open_mode |= O_CLOEXEC;
-
-	fd = open(path, open_mode, 0666);
-	if (fd < 0)
-		return NULL;
-
-	ret = fdopen(fd, mode);
-	saved_errno = errno;
-	if (!ret)
-		close(fd);
-	errno = saved_errno;
-	return ret;
-}
-
 extern struct lxc_popen_FILE *lxc_popen(const char *command)
 {
 	struct lxc_popen_FILE *fp = NULL;
@@ -1644,15 +1382,87 @@ int setproctitle(char *title)
 		if (len >= arg_end - arg_start) {
 			env_start = env_end;
 		}
+
 		arg_end = arg_start + len;
+
+		/* check overflow */
+		if (arg_end < len || arg_end < arg_start) {
+			return -1;
+		}
+
 	}
 
 	strcpy((char*)arg_start, title);
 
-	ret |= prctl(PR_SET_MM, PR_SET_MM_ARG_START,   (long)arg_start, 0, 0);
-	ret |= prctl(PR_SET_MM, PR_SET_MM_ARG_END,     (long)arg_end, 0, 0);
-	ret |= prctl(PR_SET_MM, PR_SET_MM_ENV_START,   (long)env_start, 0, 0);
-	ret |= prctl(PR_SET_MM, PR_SET_MM_ENV_END,     (long)env_end, 0, 0);
+	ret |= prctl(PR_SET_MM, PR_SET_MM_ARG_START,   arg_start, 0, 0);
+	ret |= prctl(PR_SET_MM, PR_SET_MM_ARG_END,     arg_end, 0, 0);
+	ret |= prctl(PR_SET_MM, PR_SET_MM_ENV_START,   env_start, 0, 0);
+	ret |= prctl(PR_SET_MM, PR_SET_MM_ENV_END,     env_end, 0, 0);
 
 	return ret;
 }
+
+/*
+ * Mount a proc under @rootfs if proc self points to a pid other than
+ * my own.  This is needed to have a known-good proc mount for setting
+ * up LSMs both at container startup and attach.
+ *
+ * @rootfs : the rootfs where proc should be mounted
+ *
+ * Returns < 0 on failure, 0 if the correct proc was already mounted
+ * and 1 if a new proc was mounted.
+ */
+int mount_proc_if_needed(const char *rootfs)
+{
+	char path[MAXPATHLEN];
+	char link[20];
+	int linklen, ret;
+	int mypid;
+
+	ret = snprintf(path, MAXPATHLEN, "%s/proc/self", rootfs);
+	if (ret < 0 || ret >= MAXPATHLEN) {
+		SYSERROR("proc path name too long");
+		return -1;
+	}
+	memset(link, 0, 20);
+	linklen = readlink(path, link, 20);
+	mypid = (int)getpid();
+	INFO("I am %d, /proc/self points to '%s'", mypid, link);
+	ret = snprintf(path, MAXPATHLEN, "%s/proc", rootfs);
+	if (linklen < 0) /* /proc not mounted */
+		goto domount;
+	if (atoi(link) != mypid) {
+		/* wrong /procs mounted */
+		umount2(path, MNT_DETACH); /* ignore failure */
+		goto domount;
+	}
+	/* the right proc is already mounted */
+	return 0;
+
+domount:
+	if (mount("proc", path, "proc", 0, NULL))
+		return -1;
+	INFO("Mounted /proc in container for security transition");
+	return 1;
+}
+
+int null_stdfds(void)
+{
+	int fd, ret = -1;
+
+	fd = open("/dev/null", O_RDWR);
+	if (fd < 0)
+		return -1;
+
+	if (dup2(fd, 0) < 0)
+		goto err;
+	if (dup2(fd, 1) < 0)
+		goto err;
+	if (dup2(fd, 2) < 0)
+		goto err;
+
+	ret = 0;
+err:
+	close(fd);
+	return ret;
+}
