@@ -117,12 +117,6 @@ struct lxc_attach_python_payload {
 
 static int lxc_attach_python_exec(void* _payload)
 {
-    /* This function is the first one to be called after attaching to a
-     * container. As lxc_attach() calls fork() PyOS_AfterFork should be called
-     * in the new process if the Python interpreter will continue to be used.
-     */
-    PyOS_AfterFork();
-
     struct lxc_attach_python_payload *payload =
         (struct lxc_attach_python_payload *)_payload;
     PyObject *result = PyObject_CallFunctionObjArgs(payload->fn,
@@ -353,14 +347,7 @@ LXC_get_global_config_item(PyObject *self, PyObject *args, PyObject *kwds)
 static PyObject *
 LXC_get_version(PyObject *self, PyObject *args)
 {
-    const char *rv = NULL;
-
-    rv = lxc_get_version();
-    if (!rv) {
-        return PyUnicode_FromString("");
-    }
-
-    return PyUnicode_FromString(rv);
+    return PyUnicode_FromString(lxc_get_version());
 }
 
 static PyObject *
@@ -414,10 +401,6 @@ LXC_list_containers(PyObject *self, PyObject *args, PyObject *kwds)
     /* Generate the tuple */
     list = PyTuple_New(list_count);
     for (i = 0; i < list_count; i++) {
-        if (!names[i]) {
-            continue;
-        }
-
         PyTuple_SET_ITEM(list, i, PyUnicode_FromString(names[i]));
         free(names[i]);
     }
@@ -460,9 +443,7 @@ Container_init(Container *self, PyObject *args, PyObject *kwds)
     self->container = lxc_container_new(name, config_path);
     if (!self->container) {
         Py_XDECREF(fs_config_path);
-
-        PyErr_Format(PyExc_RuntimeError, "%s:%s:%d: error during init for container '%s'.",
-            __FUNCTION__, __FILE__, __LINE__, name);
+        fprintf(stderr, "%d: error creating container %s\n", __LINE__, name);
         return -1;
     }
 
@@ -484,14 +465,8 @@ Container_new(PyTypeObject *type, PyObject *args, PyObject *kwds)
 static PyObject *
 Container_config_file_name(Container *self, void *closure)
 {
-    char *rv = NULL;
-
-    rv = self->container->config_file_name(self->container);
-    if (!rv) {
-        return PyUnicode_FromString("");
-    }
-
-    return PyUnicode_FromString(rv);
+    return PyUnicode_FromString(
+                self->container->config_file_name(self->container));
 }
 
 static PyObject *
@@ -523,10 +498,6 @@ Container_init_pid(Container *self, void *closure)
 static PyObject *
 Container_name(Container *self, void *closure)
 {
-    if (!self->container->name) {
-        return PyUnicode_FromString("");
-    }
-
     return PyUnicode_FromString(self->container->name);
 }
 
@@ -543,18 +514,71 @@ Container_running(Container *self, void *closure)
 static PyObject *
 Container_state(Container *self, void *closure)
 {
-    const char *rv = NULL;
-
-    rv = self->container->state(self->container);
-
-    if (!rv) {
-        return PyUnicode_FromString("");
-    }
-
-    return PyUnicode_FromString(rv);
+    return PyUnicode_FromString(self->container->state(self->container));
 }
 
 /* Container Functions */
+static PyObject *
+Container_attach_interface(Container *self, PyObject *args, PyObject *kwds)
+{
+    static char *kwlist[] = {"src_ifname", "dst_ifname", NULL};
+    char *src_name = NULL;
+    char *dst_name = NULL;
+    PyObject *py_src_name = NULL;
+    PyObject *py_dst_name = NULL;
+
+    if (! PyArg_ParseTupleAndKeywords(args, kwds, "O&|O&", kwlist,
+                                      PyUnicode_FSConverter, &py_src_name,
+                                      PyUnicode_FSConverter, &py_dst_name))
+        return NULL;
+
+    if (py_src_name != NULL) {
+        src_name = PyBytes_AS_STRING(py_src_name);
+        assert(src_name != NULL);
+    }
+
+    if (py_dst_name != NULL) {
+        dst_name = PyBytes_AS_STRING(py_dst_name);
+        assert(dst_name != NULL);
+    }
+
+    if (self->container->attach_interface(self->container, src_name,
+					dst_name)) {
+        Py_XDECREF(py_src_name);
+        Py_XDECREF(py_dst_name);
+        Py_RETURN_TRUE;
+    }
+
+    Py_XDECREF(py_src_name);
+    Py_XDECREF(py_dst_name);
+    Py_RETURN_FALSE;
+}
+
+static PyObject *
+Container_detach_interface(Container *self, PyObject *args, PyObject *kwds)
+{
+    static char *kwlist[] = {"ifname", NULL};
+    char *ifname = NULL;
+    PyObject *py_ifname = NULL;
+
+    if (! PyArg_ParseTupleAndKeywords(args, kwds, "O&", kwlist,
+                                      PyUnicode_FSConverter, &py_ifname))
+        return NULL;
+
+    if (py_ifname != NULL) {
+        ifname = PyBytes_AS_STRING(py_ifname);
+        assert(ifname != NULL);
+    }
+
+    if (self->container->detach_interface(self->container, ifname, NULL)) {
+        Py_XDECREF(py_ifname);
+        Py_RETURN_TRUE;
+    }
+
+    Py_XDECREF(py_ifname);
+    Py_RETURN_FALSE;
+}
+
 static PyObject *
 Container_add_device_node(Container *self, PyObject *args, PyObject *kwds)
 {
@@ -915,15 +939,8 @@ Container_get_config_item(Container *self, PyObject *args, PyObject *kwds)
 static PyObject *
 Container_get_config_path(Container *self, PyObject *args, PyObject *kwds)
 {
-    const char *rv = NULL;
-
-    rv = self->container->get_config_path(self->container);
-
-    if (!rv) {
-        return PyUnicode_FromString("");
-    }
-
-    return PyUnicode_FromString(rv);
+    return PyUnicode_FromString(
+                self->container->get_config_path(self->container));
 }
 
 static PyObject *
@@ -987,11 +1004,6 @@ Container_get_interfaces(Container *self)
     /* Add the entries to the tuple and free the memory */
     i = 0;
     while (interfaces[i]) {
-        if (!interfaces[i]) {
-            i++;
-            continue;
-        }
-
         PyObject *unicode = PyUnicode_FromString(interfaces[i]);
         if (!unicode) {
             Py_DECREF(ret);
@@ -1047,11 +1059,6 @@ Container_get_ips(Container *self, PyObject *args, PyObject *kwds)
     /* Add the entries to the tuple and free the memory */
     i = 0;
     while (ips[i]) {
-        if (!ips[i]) {
-            i++;
-            continue;
-        }
-
         PyObject *unicode = PyUnicode_FromString(ips[i]);
         if (!unicode) {
             Py_DECREF(ret);
@@ -1523,6 +1530,18 @@ static PyGetSetDef Container_getseters[] = {
 };
 
 static PyMethodDef Container_methods[] = {
+    {"attach_interface", (PyCFunction)Container_attach_interface,
+     METH_VARARGS|METH_KEYWORDS,
+     "attach_interface(src_ifname, dest_ifname) -> boolean\n"
+     "\n"
+     "Pass a new network device to the container."
+    },
+    {"detach_interface", (PyCFunction)Container_detach_interface,
+     METH_VARARGS|METH_KEYWORDS,
+     "detach_interface(ifname) -> boolean\n"
+     "\n"
+     "detach a network device from the container."
+    },
     {"add_device_node", (PyCFunction)Container_add_device_node,
      METH_VARARGS|METH_KEYWORDS,
      "add_device_node(src_path, dest_path) -> boolean\n"
